@@ -1,5 +1,5 @@
 local client = require 'client'
 
 return function (data)
-    client.setConfig { data }
+    client.setConfig(data)
 end
